/***************************************************************************
 * CVSID: $Id: main.h,v 1.5 2005/07/29 20:32:57 david Exp $
 *
 * main.h : common functions for sysfs-agent on Linux 2.6
 *
 * Copyright (C) 2003 David Zeuthen, <david@fubar.dk>
 *
 * Licensed under the Academic Free License version 2.0
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 **************************************************************************/

#ifndef MAIN_H
#define MAIN_H

#include <limits.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>
#include <libsysfs.h>
#include <libhal/libhal.h>

/*  @ingroup  HalAgentsLinux
 *  @{
 */

/** Timeout in seconds for waiting for a sysfs device to appear as
 *  a HAL device. Usually used when hotplugging usb-storage since
 *  many HAL devices (usb, usbif, scsi_host, scsi_device, block*2)
 *  appear and the linux kernel gives us these add events out of
 *  order.
 *
 *  My old PIII 650Mhz, 128MB laptop with GNOME 2.4 is slow when
 *  attaching a whole subtree of USB devices, so a value of 30 seconds 
 *  is actually reasonable. It's important to note that it's not
 *  HAL that is taking the lions share of the resources; rather it's
 *  the shell-scripts in linux-hotplug.
 */
#define HAL_LINUX_HOTPLUG_TIMEOUT 30

/** Macro to abort the program.
 *
 *  @param  expr                Format line and arguments
 */
#define DIE(expr) do {printf("*** [DIE] %s:%s():%d : ", __FILE__, __FUNCTION__, __LINE__); printf expr; printf("\n"); exit(1); } while(0)


extern dbus_bool_t is_probing;

double parse_double(const char* str);
dbus_int32_t parse_dec(const char* str);
dbus_int32_t parse_hex(const char* str);

long int find_num(char* pre, char* s, int base);
double find_double(char* pre, char* s);
int find_bcd2(char* pre, char* s);
char* find_string(char* pre, char* s);


/** Type for function to compute the UDI (unique device id) for a given
 *  HAL device.
 *
 *  @param  udi                 Unique device id of tempoary device object
 *  @param  append_num          Number to append to name if not -1
 *  @return                     New unique device id; only good until the
 *                              next invocation of this function
 */
typedef char* (*ComputeFDI)(const char* udi, int append_num);

char* rename_and_maybe_add(const char* udi, 
                           ComputeFDI naming_func,
                           const char* namespace);

char* find_udi_from_sysfs_path(const char* path, 
                               int max_time_to_try);

char* find_parent_udi_from_sysfs_path(const char* path, 
                                      int max_time_to_try);

const char* get_last_element(const char* s);

char* read_single_line(char* filename_format,...);

void find_and_set_physical_device(char* udi);

char* find_udi_by_key_value(const char* key,
                            const char* value,
                            int max_time_to_try);

const char* drivers_lookup(const char* device_path);
void drivers_collect(const char* bus_name);

extern LibHalFunctions hal_functions;

extern DBusConnection* dbus_connection;

extern char sysfs_mount_path[SYSFS_PATH_MAX];

/* @} */

#endif /* MAIN_H */
